package com.ruoyi.framework.websocket;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 *
 * 前后端交互的类实现消息的接收推送(自己发送给所有人(不包括自己))
 *
 */
@ServerEndpoint(value = "/websocket/message/{userId}")
@Component
public class OneToManyWebSocket {
    /**
     * WebSocketUsers 日志控制器
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(WebSocketUsers.class);

    /** 记录当前在线连接数 */
    private static AtomicInteger onlineCount = new AtomicInteger(0);

    /** 存放所有在线的客户端 key 为用户Id */
    private static final Map<String, Session> clients = new ConcurrentHashMap<>();

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session,@PathParam("userId") String userId) {
        if(clients.containsKey(userId)){
            clients.remove(userId);
            clients.put(userId, session);
        }else{
            onlineCount.incrementAndGet(); // 在线数加1
            clients.put(userId, session);
            //sendMessage("你好啊客户端"+userId,session);
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session) {
        for (String userId : clients.keySet()) {
            if(clients.get(userId).equals(session)){
                clients.remove(session);
                onlineCount.decrementAndGet(); // 在线数减1
            }
        }
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message
     *            客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        LOGGER.info("服务端收到客户端[{}]的消息:{}", session.getId(), message);
        //this.sendMessage(message, session);
    }

    @OnError
    public void onError(Session session, Throwable error) {
        LOGGER.error("发生错误");
        error.printStackTrace();
    }

    /**
     * 获取在线用户列表
     *
     * @return 返回用户集合
     */
    public static Map<String, Session> getUsers()
    {
        return clients;
    }


    /**
     * 群发消息
     * @param message
     * @param userId
     */
    public static void sendMessage(String message,String userId) {
        for (Map.Entry<String, Session> sessionEntry : clients.entrySet()) {
            Session toSession = sessionEntry.getValue();
            String suid = sessionEntry.getKey();
            // 排除掉自己
            if (!suid.equals(userId)) {
                LOGGER.info("服务端给客户端[{}]发送消息{}", toSession.getId(), message);
                toSession.getAsyncRemote().sendText(message);
            }
        }
    }

    /**
     * 群发消息
     * @param message
     * @param userIds
     */

    public static void sendMessageByUserIds(String message, List<String> userIds) {

        for (String id : userIds) {
            Session session = clients.get(id);
            if(session!=null){
                LOGGER.info("服务端给客户端[{}]发送消息{}", session.getId(), message);
                try {
                    session.getAsyncRemote().sendText(message);
                }catch (Exception e){
                    LOGGER.info("数据发送失败！疑似断开连接", session.getId(), message);
                    clients.remove(id);
                }
            }
        }
    }

    public Session getUserSession(String userId){
        return clients.get(userId);
    }
}
